from django.http import HttpResponse, HttpResponseBadRequest
from django.core.files.storage import default_storage
from django.views.decorators.csrf import csrf_exempt
from django.utils import simplejson as json

from directupload.backends import get_uploadify_backend

from urlparse import parse_qsl
import os

def uploadify_options_view(request):
    if not request.POST:
        return HttpResponseBadRequest()
    uploadify_options = {}
    if 'upload_to' in request.POST:
        uploadify_options['folder'] = request.POST['upload_to']
    backend = get_uploadify_backend()
    data = backend(request=request, 
                   uploadify_options=uploadify_options).get_options_json()
    return HttpResponse(data)

@csrf_exempt
def upload_file(request):
    #this is handled by a different session then the user's browser, hence csrf exempt
    if not request.POST:
        return HttpResponseBadRequest()
    from directupload.backends.djangoview import unsign
    data = dict(parse_qsl(unsign(request.POST['payload'])))
    assert data['request_time'] #TODO respect some expiration
    path = request.POST.get('targetpath', os.path.join(data['upload_to'], request.POST['Filename']))
    file_path = default_storage.save(path, request.FILES['Filedata']) #TODO how to tell the storage engine not to rename?
    return HttpResponse(file_path)

def determine_name(request):
    if not request.POST:
        return HttpResponseBadRequest()
    desired_path = os.path.join(request.POST['upload_to'], request.POST['filename'])
    path = default_storage.get_available_name(desired_path)
    data = {'targetpath':path,
            'targetname':os.path.split(path)[-1],}
    backend = get_uploadify_backend()
    backend(request=request, uploadify_options={'folder':request.POST['upload_to']}).update_post_params(data)
    return HttpResponse(json.dumps(data))




from django.conf import settings

UPLOADIFY_OPTIONS = ('auto', 'buttonImg', 'buttonText', 'cancelImg', 'checkScript', 'displayData', 'expressInstall', 'fileDataName', 'fileDesc', 'fileExt', 'folder', 'height', 'hideButton', 'method', 'multi', 'queueID', 'queueSizeLimit', 'removeCompleted', 'rollover', 'script','scriptAccess', 'scriptData', 'simUploadLimit', 'sizeLimit', 'uploader', 'width', 'wmode')

UPLOADIFY_METHODS = ('onAllComplete', 'onCancel', 'onCheck', 'onClearQueue', 'onComplete', 'onError', 'onInit', 'onOpen', 'onProgress', 'onQueueFull', 'onSelect', 'onSelectOnce', 'onSWFReady')

BUTTON_TEXT = 'Select File'

# Defaults for required Uploadify options
DEFAULT_CANCELIMG = settings.STATIC_URL + "uploadify/uploadify-cancel.png"
DEFAULT_UPLOADER  = settings.STATIC_URL + "uploadify/uploadify.swf"



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse

from django.utils import simplejson as json

from common import UPLOADIFY_OPTIONS, UPLOADIFY_METHODS, DEFAULT_CANCELIMG, DEFAULT_UPLOADER, BUTTON_TEXT

class BaseUploadifyBackend(object):
    def __init__(self, request, uploadify_options={}, post_data={}):
        self.request = request
        self.options = getattr(settings, 'UPLOADIFY_DEFAULT_OPTIONS', {})
        self.options.update(uploadify_options)
        
        if any(True for key in self.options if key not in UPLOADIFY_OPTIONS + UPLOADIFY_METHODS):
            raise ImproperlyConfigured("Attempted to initialize with unrecognized option '%s'." % key)
        
        _set_default_if_none(self.options, 'cancelImage', DEFAULT_CANCELIMG)
        _set_default_if_none(self.options, 'swf', DEFAULT_UPLOADER)
        _set_default_if_none(self.options, 'uploader', self.get_uploader())
        _set_default_if_none(self.options, 'buttonText', BUTTON_TEXT)
        _set_default_if_none(self.options, 'checkExisting', self.get_check_existing())
        _set_default_if_none(self.options, 'determineName', self.get_determine_name())
        
        self.post_data = post_data
        self.build_post_data()
    
    def get_check_existing(self):
        return False
    
    def get_determine_name(self):
        return reverse('uploadify-determine-name')
    
    def get_uploader(self):
        pass
    
    def build_post_data(self):
        pass
    
    def update_post_params(self, params):
        pass
    
    def get_options_json(self):
        self.options['postData'] = self.post_data
        subs = []
        for key in self.options:
            if key in UPLOADIFY_METHODS:
                subs.append(('"%%%s%%"' % key, self.options[key]))
                self.options[key] = "%%%s%%" % key
                
        out = json.dumps(self.options)
        
        for search, replace in subs:
            out = out.replace(search, replace)
            
        return out

def _set_default_if_none(dict, key, default=None):
    if key not in dict:
        dict[key] = default

